/********************************************************************
Beschreibung:
Zwei Pollin-Evaluationboards mit jeweils einem RFM12 um miteineander 
zu kommunizieren. Auf den Boards wurden ATmega8 mit 8 MHz internen 
Takt eingesetzt.

Dieser Code ist auf beiden Board drauf.

Pinbelegung
-----------------------
ATMEGA8           RFM12
-----------------------
SCK               SCK
MISO              SDO
MOSI              SDI
SS                nSEL
PD2 (INT0)        nIRQ
PD5               LED1
PD6               LED2
VDD               VDD
PD7               FSK
PD3 (INT1)        FFIT
PB0               VDI
**********************************************************************/

#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <stdlib.h>
#include <compat/deprecated.h>


#define F_CPU 8000000UL
#define BAUD 19200UL

#define LED1_ON		PORTD |= (1 << PD5)
#define LED2_ON		PORTD |= (1 << PD6)
#define LED1_OFF	PORTD &= ~(1 << PD5)
#define LED2_OFF	PORTD &= ~(1 << PD6)
#define LED1_TOG	PORTD ^= (1 << PD5)
#define LED2_TOG	PORTD ^= (1 << PD6)

#define Taster3		PIND  &  (1<<PIND4)

// Funktionsprototypen
void rfm12_write_data(uint16_t);			
uint16_t rfm12_write(uint16_t);
uint8_t rfm12_read();	
void port_init();		
void spi_master_init();						
void rfm12_init();						
void rfm12_rx_on();
void rfm12_tx_on();
void uart_init();
uint8_t uart_get_byte();
void uart_send_byte(uint8_t);
void uart_send_int(char*);


uint8_t rfm_packet_len, rfm_empfangsbyte, rfm12_fifo_voll;	// Globale Variablen

int main(void)
{
	uint8_t i=0;											// Zhlervariable	

	port_init();											// Ein/Ausgnge initialisieren
	spi_master_init();										// SPI initialisieren
	uart_init();											// USART initialisieren
	rfm12_init();											// RFM12 initialisieren

	sei();													// Interrupts aktivieren

	rfm12_rx_on();											// Empfnger einschalten
	
	while(1)												// Arbeitsschleife
	{
		if(rfm12_fifo_voll==1)
		{
			rfm_packet_len=rfm12_read();					// Erstes Nutzerbyte ist die Packetlnge
			for(uint8_t i=1;i<=rfm_packet_len;i++)			// Restlichen Nutzerbytes ...
			{
				rfm_empfangsbyte=rfm12_read();				// ... auslesen	
				uart_send_byte(rfm_empfangsbyte);			// Nutzerbyte ber UART an den PC schicken
				if(rfm_empfangsbyte=='H')LED2_ON;			// LED2 an wenn Zeichen 'H' empfangen wird
				if(rfm_empfangsbyte=='W')LED2_OFF;			// LED2 bei Zeichen 'W' wieder lschen
			}
			rfm12_write(0xCA81);							// FIFO deinitialisieren
			rfm12_write(0xCA83);							// FIFO deaktivieren
			rfm12_fifo_voll=0;								// Interrupt-Flag zurcksetzen
		}

		if(Taster3)											// Wenn Taster3 gedrckt wird...
		{
			_delay_ms(100);									// ... 100ms warten (Tastenentprellung)
			if(Taster3)										// Wenn Taster3 noch gedrckt ...
			{
				i++;										// Zhlervariable inkrementieren
				rfm12_tx_on();								// Sender einschalten
				if(i==1)
				{
					rfm12_write_data(0xB8AA);				// Prambel senden			
					rfm12_write_data(0xB8AA);				// Prambel senden						
					rfm12_write_data(0xB8AA);				// Prambel senden						
					rfm12_write_data(0xB82D);				// Syncronmuster senden			
					rfm12_write_data(0xB8D4);				// Syncronmuster senden				
					rfm12_write_data(0xB800 | 6 );			// Nutzerbyte senden							
					rfm12_write_data(0xB800 | 'H');			// Nutzerbyte senden							
					rfm12_write_data(0xB800 | 'e');			// Nutzerbyte senden							
					rfm12_write_data(0xB800 | 'l');			// Nutzerbyte senden							
					rfm12_write_data(0xB800 | 'l');			// Nutzerbyte senden							
					rfm12_write_data(0xB800 | 'o');			// Nutzerbyte senden							
					rfm12_write_data(0xB800 | ' '); 		// Nutzerbyte senden	
					rfm12_write_data(0xB8AA);				// Dummybyte
				}

				if(i==2)
				{
					rfm12_write_data(0xB8AA);				// Prambel senden			
					rfm12_write_data(0xB8AA);				// Prambel senden						
					rfm12_write_data(0xB8AA);				// Prambel senden						
					rfm12_write_data(0xB82D);				// Syncronmuster senden			
					rfm12_write_data(0xB8D4);				// Syncronmuster senden				
					rfm12_write_data(0xB800 | 6 );			// Nutzerbyte senden							
					rfm12_write_data(0xB800 | 'W');			// Nutzerbyte senden							
					rfm12_write_data(0xB800 | 'o');			// Nutzerbyte senden							
					rfm12_write_data(0xB800 | 'r');			// Nutzerbyte senden							
					rfm12_write_data(0xB800 | 'l');			// Nutzerbyte senden							
					rfm12_write_data(0xB800 | 'd');			// Nutzerbyte senden							
					rfm12_write_data(0xB800 | ' '); 		// Nutzerbyte senden	
					rfm12_write_data(0xB8AA);				// Dummybyte
				}					
				if(i==2)i=0;								// Wenn Zhler auf 2 dann wieder auf 0 setzen
				
				rfm12_write_data(0x8208);					// Sender ausschalten
				rfm12_rx_on();								// Empfnger einschalten
			}
		}				
	}
}


void port_init()
{
	DDRD |= (1 << PD5) | (1 << PD6);						// PD5 (Led1), PD6 (Led2) als Ausgang		
	DDRD  &= ~(1<<PIND4);									// PD4 (Taster3) als Eingang
	DDRD  |=  (1<<PD7);										// PD7 (nFFS) als Ausgang
 	PORTD |=  (1<<PD7);										// nFFS auf high ziehen
	
	DDRD  &= ~(1<<PIND2);					 				// PD2(INT0) fr den nIRQ-Pin	
	DDRD  &= ~(1<<PIND3);				 					// PD3(INT1) fr den FIFO-Interrupt

	MCUCR |= (1<<ISC10) | (1<<ISC11);						// INT1 steigende Flanke
	GICR  |= (1<<INT1);										// INT1 aktivieren
}

void spi_master_init (void) 
{
	uint8_t status;
	DDRB = (1<<PB2) | (1<<PB3) | (1<<PB5);					// setze SCK,MOSI,PB2 (SS) als Ausgang
	DDRB &= ~(1<<PB4);										// setze MISO als Eingang
	PORTB = (1<<PB5) | (1<<PB2);							// SCK und PB2 high (ist mit SS am Slave verbunden)
	SPCR = (1<<SPE) | (1<<MSTR);							// Aktivierung des SPI, Master, Taktrate fck/4
	status = SPSR;											// Status lschen
}

ISR(INT1_vect)												// FIFO-Interrupt (PIN: FFIT)
{
	rfm12_fifo_voll=1;										// Interrupt-Flag setzen
}

// *********************************************************************************************** //
// ** 									RFM12-Funktionen										** //
// *********************************************************************************************** //

void rfm12_init(void)
{
	_delay_ms(100);											// Warten bis Power-On-Sequenz vom RFM12 abgeschlossen ist
  	rfm12_write(0x80D8);									// TX-FIFO, RX-FIFO, Band = 433 MHz, Quarz = 12,5 pF 	
	rfm12_write(0x82D1);									// Empfnger, Basis-Band, Synthesizer, Kein Takt-Ausgang
  	rfm12_write(0xA140);									// Sendefrequenz = 430.8 MHz
	rfm12_write(0xC623);									// 9600 Bit/s
	rfm12_write(0x9480);									// VDI, Schnell, 200 KHz, 0 dB, -103 dB
  	rfm12_write(0xC2AC);									// Auto-Lock, Digitalfilter, mittlere Schwelle
  	rfm12_write(0xCA81);									// FIFO = 8 Bit, Synchronmuster, FIFO leeren/sperren = Empfnger, Reset-Schaltung aus
  	rfm12_write(0xC4F7);									// Auto-Mode = VDI unabhngig, Range-Limit = -4/+3, Fine-Mode, Offset aktiv, AFC aktiv
  	rfm12_write(0x9800);									// Modulation-Polarity = normal, Frequenzhub = 15 KHz, Ausgangsleistung = Maximal
  	rfm12_write(0x0000);									// Status Register lesen (Interrupts werden gelscht)
	rfm12_write(0xCA81);									// FIFO initialisieren
	rfm12_write(0xCA83);									// FIFO aktivieren
}

void rfm12_write_data(uint16_t wert)
{
	while(PIND & (1<<PIND2));								// Warten bis Modul bereit
	rfm12_write(wert);										// Sendeword bertragen
}

uint16_t rfm12_write(uint16_t wert)
{
	uint16_t newHigh=0,newLow=0,newWord=0;				
	PORTB &= ~(1<<PB2);										// SS am Slave Low --> Beginn der bertragung
	SPDR = wert >> 8;										// High-Byte bertragen
	while (!(SPSR & (1<<SPIF)));							// Warten bis bertragung abgeschlossen
	newHigh = SPDR;											// Empfangenes High-Byte lesen
	SPDR = wert;											// Low-Byte bertragen
	while (!(SPSR & (1<<SPIF)));							// Warten bis bertragung abgeschlossen
	newLow = SPDR;											// Empfangenes Low-Byte lesen
	PORTB |= (1<<PB2);										// SS am Slave High --> Ende der bertragung
	newWord = (newHigh<<=8) | newLow;						// High-Byte und Low-Byte zusammenfhren	
	return newWord;											// Rckgabe des empfangenen Words
}

uint8_t rfm12_read(void)
{
	uint16_t FIFO_data;							
	rfm12_write(0x0000);									// Status Register lesen (Interrupts werden gelscht)
	while(PIND & (1<<PIND2));								// Warten bis Modul bereit
	FIFO_data=rfm12_write(0xB000);							// FIFO-Daten einlesen (16Bit)
	return(FIFO_data&0x00FF);								// LOW-Byte der FIFO-Daten zurckgeben (Datenbyte)
}

void rfm12_rx_on()
{
	rfm12_write(0x82C8);									// Empfnger einschalten
	_delay_ms(1);											// Warten bis Empfnger an ist
	rfm12_write(0xCA81);									// FIFO initialisieren
	rfm12_write(0xCA83);									// FIFO aktivieren
}

void rfm12_tx_on()
{
	rfm12_write(0x8208);									// Empfnger ausschalten
	_delay_ms(1);											// Warten bis Empfnger aus ist
	rfm12_write(0x0000);									// Status Register lesen (Interrupts werden gelscht)
	rfm12_write(0x8271);									// Sender aktivieren
}

// *********************************************************************************************** //
// ** 									USART-Funktionen										** //
// *********************************************************************************************** //

ISR(USART_RXC_vect)											// USART Empfangs-Interrupt
{
	uint8_t daten;
	daten = uart_get_byte();								// Zeichen aus Puffer holen
}

void uart_init(void)
{
	uint8_t x;
	UBRRL = (F_CPU / (16ul * BAUD))-1;						// Baudrate mit Takt und Baud
	UCSRB |= (1 << TXEN) | (1 << RXEN); 					// Sender und Empnger aktivieren
	UCSRC |= (1 << URSEL) | (1 << UCSZ1) | (1 << UCSZ0);	// async 8bit
	x = UDR;												// Empfnger leeren
	sbi(UCSRB,RXCIE );										// USART-Empfangs-Interrupt aktivieren
}

uint8_t uart_get_byte(void)
{
	while(!(UCSRA & (1 << RXC)));							// warten bis Empfang abgeschlossen
	return UDR;												// Byte abholen
}

void uart_send_byte(uint8_t x)
{
	while(!(UCSRA & (1 << UDRE)));							// warten bis Schnittstelle bereit
	UDR = x;												// Zu sendendes Byte bergeben
}

void uart_send_int(char *s)
{
    while (*s)												// so lange *s ungleich '\0' (String-Endezeichen)
    {   
        uart_send_byte(*s);									// Byte senden
        s++;												// nchstes Byte whlen
    }
	uart_send_byte('\r');									// Carriage Return senden
}
